/***********************************************************************
 * copyright (c) 2005 Jan Demter                                   *
 *                                                                     *
 * This program is free software; you can redistribute it and/or       *
 * modify it under the terms of the GNU General Public License         *
 * as published by the Free Software Foundation; either version 2      *
 * of the License, or (at your option) any later version.              *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,              *
 * MA  02111-1307, USA.                                                *
 *                                                                     *
 * ------------------------------------------------------------------- *
 * project : GONE NSLP                                               *
 * author  : Jan Demter <demter@math.uni-goettingen.de>           * 
 ***********************************************************************/
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
//#include <sys/sendfile.h>

#include "GONE.h"
#include "GONEClient.h"

//#include "objects/GONEClientSendMessage.h"
#include "objects/GONEHeader.h"

GONEClient::GONEClient()
{
	sock = 0;
}

GONEClient::~GONEClient()
{
}

void GONEClient::connectToDaemon()
{
	struct sockaddr_un _serv_addr;
	int len;

	memset((char *)&_serv_addr, 0, sizeof(_serv_addr));
	_serv_addr.sun_family = AF_UNIX;
	strcpy(_serv_addr.sun_path, NSLP_PATH_TO_UNIX_SOCKET);
	len = sizeof(_serv_addr);

	if ((sock = socket(AF_UNIX, SOCK_STREAM,0)) < 0) {
		perror("GONE: socket creation failed");
		exit(-1);
	}

	if (connect(sock, (struct sockaddr *) &_serv_addr, len) < 0) {
		perror("GONE: connect to socket failed");
	 	exit(-1);
	}
}

// a dead-simple testcase
void GONEClient::sendCommand(unsigned int cmd)
{

	char * daten = "This is to certify that there was actually any data sent.";
	if(cmd == GONE_CLIENT_SENDMESSAGE){
		GONEHeader message_header = GONEHeader(GONE_CLIENT_SENDMESSAGE, strlen(daten)+1,
											   inet_addr("192.168.2.11"), 0);
		
		int len = message_header.getLength()+strlen(daten)+1;
		char * buffer = (char *) malloc(len);
		
		memcpy(buffer, message_header.getBuffer(), message_header.getLength());
		memcpy(buffer+message_header.getLength(), daten, strlen(daten)+1);
		
		if(len != send(sock, buffer, len, 0)){
			printf("GONEClient::sendCommand: send() kaputt\n");
		}
		
		free(buffer);
		
	}
	
	
}

// send some data
void GONEClient::sendData(unsigned int addr, char * data, unsigned int length)
{
	
	GONEHeader message_header = GONEHeader(GONE_CLIENT_SENDMESSAGE, length,
										   addr, 0);
	
	int len = message_header.getLength()+length;
	char * buffer = (char *) malloc(len);
	
	memcpy(buffer, message_header.getBuffer(), message_header.getLength());
	memcpy(buffer+message_header.getLength(), data, length);
	
	if(len != send(sock, buffer, len, 0)){
		std::cerr<<"GONEClient::sendCommand: incomplete transfer\n";
	}
	
	free(buffer);
	
}

int GONEClient::getData(unsigned int addr, char * data, unsigned int length)
{
	//FIXME, this does not work at all yet
	
	
	GONEHeader message_header = GONEHeader(GONE_CLIENT_GETMESSAGE, length,
										   addr, 0);
	
	int len = message_header.getLength()+length;
	char * buffer = (char *) malloc(len);
	
	memcpy(buffer, message_header.getBuffer(), message_header.getLength());
	memcpy(buffer+message_header.getLength(), data, length);
	
	if(len != recv(sock, buffer, len, 0)){
		std::cerr<<"GONEClient::sendCommand: incomplete transfer\n";
		
	}
	
	return -23;
	
}

/*
void GONEClient::sendCommand(unsigned int cmd, char * file)
{
	
	if(cmd == GONE_CLIENT_SENDMESSAGE){
		
		int fd = open(file, O_RDONLY, 0);
		if(fd < 0)
			exit(23);
		
		struct stat finfo;
		
		if(fstat(fd, &finfo) > 0)
			exit(123);
		
		GONEHeader message_header = GONEHeader(GONE_CLIENT_SENDMESSAGE, finfo.st_size,
											   inet_addr("192.168.2.11"), 0);
		off_t offset = 0;
		
		// FIXME: this will not result in one gist message, use some magic in the server or whatnot...
		send(sock, message_header.getBuffer(), message_header.getLength(), 0);
		sendfile(sock, fd, &offset, finfo.st_size);
	}
	
	
}
 */
// ends the connection
void GONEClient::sendFin(unsigned int addr)
{
	
	GONEHeader message_header = GONEHeader(GONE_CLIENT_FIN, 0,
										   addr, 0);
	
	if(message_header.getLength() != send(sock, message_header.getBuffer(),  message_header.getLength(), 0)){
		std::cerr<<"GONEClient::sendFin: incomplete transfer\n";
	}
	
}
