/***********************************************************************
 * Copyright (c) 2005 Jan Demter                                   *
 *                                                                     *
 * This program is free software; you can redistribute it and/or       *
 * modify it under the terms of the GNU General Public License         *
 * as published by the Free Software Foundation; either version 2      *
 * of the License, or (at your option) any later version.              *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,              *
 * MA  02111-1307, USA.                                                *
 *                                                                     *
 * ------------------------------------------------------------------- *
 * project : GONE NSLP                                               *
 * author  : Jan Demter <demter@math.uni-goettingen.de>           * 
 ***********************************************************************/

#ifndef _GONE_SERVER_H_
#define _GONE_SERVER_H_

#define GONE_VERBOSE(b) if(verbose){b}else{}

#include <cstring>

#include "../nslp_api/NslpApi.h"
#include "../nslp_api/NslpApiCallback.h"
#include "../../gist/GistConfiguration.h"

#include "GONEClient.h"
#include "GONE.h"
#include "GONEConnectionCapability.h"
#include "GONEConnection.h"

struct nslp_sock_t
{
	int sock;
	unsigned int sid[4];
};

class GONEServer {

public:
	GONEServer();
	GONEServer(bool verbose);
	GONEServer(unsigned int my_addr);
	virtual ~GONEServer();
	
	void run();
	int createServerSocket();

	void acceptClient(int sockfd, void * sockData);
	void handleClientSocket(int sockfd, void * sockData);

	void closeSocket(int sock);
	void addSocket(int sock);

	static void newSession(unsigned char sid[16]);
	static void printSession(unsigned char sid[16]);

	NslpApi * api;

private:
	int nslpSock;
	int clientSockCount;
	GistConfiguration gistconfig;
	unsigned int my_addr;
	int datafile;
	int urandom;
	//Holds connections initiated by us:
	HashTable<GONEConnection, GONEConnection> * connections;
	//Holds capabilities for connections to/through us
	HashTable<GONEConnectionCapability, GONEConnectionCapability> * capabilitytable;
	
	NslpApiSocketCallback<GONEServer> * clientCallback;
	NslpApiSocketCallback<GONEServer> * serverCallback;
	NslpApiCallback<GONEServer> * nslpApiCallback;

	void handleRecvMessage(api_recv_msg * s);
	void handleMessageStatus(api_msg_status *s);
	void handleNetworkNotification(api_nw_notification *s);
	bool verbose;
	
	//stats:
	bool client_cache_full;
	bool used;
};

#endif // _GONE_SERVER_H_
