<?php

$copyright = <<<COPYRIGHT
/***********************************************************************
 * Copyright (c) 2005 Henning Peters                                   *
 *                                                                     *
 * This program is free software; you can redistribute it and/or       *
 * modify it under the terms of the GNU General Public License         *
 * as published by the Free Software Foundation; either version 2      *
 * of the License, or (at your option) any later version.              *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,              *
 * MA  02111-1307, USA.                                                *
 *                                                                     *
 * ------------------------------------------------------------------- *
 * project : NAT/FW NSLP                                               *
 * author  : Henning Peters <hpeters@math.uni-goettingen.de>           * 
 ***********************************************************************/


COPYRIGHT;

if (!isset($argv[0]) || !isset($argv[1])) {
	echo 'ERROR: '.$argv[0]." xmlfile\n";
	die();
}

if (!file_exists($argv[1])) {
	echo 'ERROR: '.$argv[1]." not found\n";
	die();
}

$file = $argv[1];
$onlyobject = isset($argv[2]) ? $argv[2] : "";
$content = file_get_contents($file);
$xml = simplexml_load_string($content);


foreach ($xml->object as $object) {
	if ($onlyobject != "" && $onlyobject != $object->name) continue;
	echo $object->name."\n";

	$offset = 0;
	$res = array();
	$ptr = array();
	
	foreach ($object->elements->children() as $element) {
		//echo "offset: $offset\n";
		if ($element->name != '') {
			$ptr[floor($offset / 8)][] = $element->name;
		}
		switch($element->type) {
			case 'bool':
				$res[] = bool($object, $element, $offset);
				$offset += ((int)$element->length) ? (int)$element->length : 1;
				break;

			case 'unsigned char':
				$res[] = int($object, $element, 'unsigned char');
				$offset += ((int)$element->length) ? (int)$element->length : 8;
				break;

			case 'unsigned short':
				$res[] = int($object, $element, 'unsigned short');
				$offset += ((int)$element->length) ? (int)$element->length : 16;
				break;

			case 'unsigned int':
				$res[] = int($object, $element, 'unsigned int');
				$offset += ((int)$element->length) ? (int)$element->length : 32;
				break;

			case 'in_addr':
				$res[] = int($object, $element, 'in_addr');
				$offset += ((int)$element->length) ? (int)$element->length : 32;
				break;

			case 'in6_addr':
				$res[] = int($object, $element, 'in6_addr');
				$offset += ((int)$element->length) ? (int)$element->length : 128;
				break;

			case 'IPaddr':
				$res[] = int($object, $element, 'IPaddr');
				$offset += ((int)$element->length) ? (int)$element->length : 128;
				break;

			default: $offset += $element->length;
		}
	}
	
	$head = head($object, ceil($offset / 8), $ptr);
	file_put_contents($object->name.".cpp", $copyright.$head.join('', $res));
	file_put_contents($object->name.".h", $copyright.classdef($object, ceil($offset / 8), $ptr));
}

function classdef($object, $length, $ptr)
{
	$res = '';
	$res .= "#ifndef _".strtoupper($object->name)."_H_\n";
	$res .= "#define _".strtoupper($object->name)."_H_\n\n";
	
	if ($object->type != 'commonheader') {
		$res .= "#include \"NatFwCommonObjectHeader.h\"\n";
	}
	$res .= "#include \"GenericObject.h\"\n\n";
	if (isset($object->code)) {
		$res .= "#define ".strtoupper($object->name)." ".$object->code."\n\n";
	}

	$array = array();
	$noparams = true;
	foreach($object->elements->children() as $element) {
		if ($element->name != '') {
			$array[] = $element->type." ".$element->name;
			$noparams = false;
		}
	}
	if ($object->type != 'commonheader' && $object->type != 'nslpheader') {
		$array[] = "bool a = false, bool b = false";
	}

	$res .= "class ".$object->name." : public GenericObject\n";
	$res .= "{\npublic:\n";

	if (!$noparams) {
		$res .= "\t".$object->name."() {}\n";
	}
	$res .= "\t".$object->name."(".(isset($array) ? join(', ', $array) : '').");\n";
	$res .= "\t".$object->name."(unsigned char * buffer, unsigned int length);\n";
	$res .= "\tvirtual ~".$object->name."() {}\n\n";
	$res .= "\tvirtual void analyze(unsigned char * buffer, unsigned int length);\n\n";
	
	foreach($object->elements->children() as $element) {
		if ($element->name != '') {
			$res .= "\t".$element->type." ".$element->name."();\n";
			$res .= "\tvoid ".$element->name."(".$element->type." ".$element->name.");\n";
		}
	}
	
	$res .= "\nprivate:\n";

	foreach($object->elements->children() as $element) {
		if ($element->name != '') {
			$res .= "\tunsigned char * ".$element->name."Ptr;\n";
		}
	}

	$res .= "};\n\n";
	$res .= "#endif // _".strtoupper($object->name)."_H_\n";

	return $res;
}

function head($object, $length, $ptr)
{
	$res = '';
	$res .= "#include \"".$object->name.".h\"\n\n";

	foreach($object->elements->children() as $element) {
		if ($element->name != '')
			$array[] = $element->type." ".$element->name;
	}
	if ($object->type != 'commonheader') {
		$array[] = "bool a, bool b";
	}

	// constructor #1
	$res .= $object->name."::".$object->name."(".(isset($array) ? join(', ', $array) : '').")\n";
	$res .= "{\n";
	if ($object->type != 'commonheader') {
		$res .= "\textendBuffer(4);\n";
		$res .= "\tNatFwCommonObjectHeader ch(a, b, ".strtoupper($object->name).", ".($length/4).");\n";
		$res .= "\tmemcpy(tail, ch.getBuffer(), 4);\n";
		$res .= "\ttail += 4;\n\n";
	}
	$res .= "\textendBuffer($length);\n";
	$res .= "\tanalyze(tail, $length);\n";

	foreach ($object->elements->children() as $element) {
		if ($element->name != '') {
			$res .= "\tthis->".$element->name."(".$element->name.");\n";
			echo "\t".$element->name."\n";
		}
	}
	$res .= "}\n\n";

	// constructor #2
	$res .= $object->name."::".$object->name."(unsigned char * buffer, unsigned int length)\n";
	$res .= "{\n";
	$res .= "\tsetBuffer(buffer, length);\n";
	$res .= "}\n\n";

	// analyze
	$res .= "void ".$object->name."::analyze(unsigned char * buffer, unsigned int length)\n";
	$res .= "{\n";
	$res .= "\ttail = buffer;\n";

	foreach($ptr as $k => $p1) {
		if ($k != 0) {
			$res .= "\ttail += $k;\n";
		}
		foreach($p1 as $p2) {
			$res .= "\t".$p2."Ptr = tail;\n";
		}
	}
	$res .= "\ttail = buffer + length;\n";
	$res .= "}\n\n";
	
	return $res;
}

function int($object, $element, $mode)
{
	switch ($mode) {
		case 'unsigned int':
			$size = 4;
			$hton = 'htonl';
			$ntoh = 'ntohl';
			break;

		case 'unsigned char':
			$size = 1;
			$hton = NULL;
			$ntoh = NULL;
			break;

		case 'unsigned short':
			$size = 2;
			$hton = 'htons';
			$ntoh = 'ntohs';
			break;

		case 'in_addr':
			$size = 4;
			$hton = NULL;
			$ntoh = NULL;
			break;

		case 'in6_addr':
			$size = 16;
			$hton = NULL;
			$ntoh = NULL;
			break;
	}

	$res = $element->type." ".$object->name."::$element->name()\n";
	$res .= "{\n";
	$res .= "\t".$element->type." tmp;\n";
	$res .= "\tmemcpy(&tmp, ".$element->name."Ptr, $size);\n";
	$res .= "\treturn ".($ntoh ? $ntoh."(tmp)" : "tmp");
	$res .= isset($element->bitmask) ? " & ".$element->bitmask : "";
	$res .= ";\n}\n\n";

	$res .= "void ".$object->name."::$element->name(".$element->type." ".$element->name.")\n";
	$res .= "{\n";

	if (isset($element->bitmask)) {
		$res .= "\t".$element->type." tmp;\n";
		$res .= "\tmemcpy(&tmp, ".$element->name."Ptr, $size);\n";
		$res .= "\ttmp = tmp | ($hton(".$element->name;
		$res .= isset($element->bitmask) ? " & ".$element->bitmask : "";
		$res .= "));\n";
		$res .= "\tmemcpy(".$element->name."Ptr, &tmp, $size);\n";
	} else {
		if ($hton) {
			$res .= "\t".$element->name." = $hton(".$element->name.");\n";
		}
		$res .= "\tmemcpy(".$element->name."Ptr, &".$element->name.", $size);\n";
	}

	$res .= "}\n\n";
	return $res;
}

function bool($object, $element, $offset)
{
	$bitmask = "0x".dechex(pow(2, 15 - $offset % 8));
	$res = '';

	$res .= $element->type." ".$object->name."::$element->name()\n";
	$res .= "{\n";
	$res .= "\tshort tmp;\n";
	$res .= "\tmemcpy(&tmp, ".$element->name."Ptr, 2);\n";
	$res .= "\treturn (bool)(ntohs(tmp) & $bitmask);";
	$res .= "\n}\n\n";

	$res .= "void ".$object->name."::$element->name(".$element->type." ".$element->name.")\n";
	$res .= "{\n";
	$res .= "\tshort tmp;\n";
	$res .= "\tmemcpy(&tmp, ".$element->name."Ptr, 2);\n";
	$res .= "\ttmp = tmp | htons(".$element->name." ? $bitmask: 0x0);\n";
	$res .= "\tmemcpy(".$element->name."Ptr, &tmp, 2);\n";
	$res .= "}\n\n";
	
	return $res;
}

?>
